@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2009 Free Software Foundation
+   Copyright (C) 2009-2010 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -33,12 +33,14 @@
   BOOL         isQualified;
   NSString     *qualifier;
   NSString     *newLine;
+  NSStringEncoding encoding;
 }
 
 - (id)initWithHandle:(NSFileHandle *)fileHandle;
 - (void)setFieldNames:(NSArray *)array andWriteIt:(BOOL)flag;
 - (void)writeDataSet:(NSArray *)array;
 - (NSString *)formatOneLine:(NSArray *)values;
+- (void)setStringEncoding: (NSStringEncoding) enc;
 
 @end
 
