@@ -28,7 +28,6 @@
 
 
 
-
 @implementation DBSoap
 
 - (void)login :(NSURL *)url :(NSString *)userName :(NSString *)password
@@ -53,8 +52,6 @@
       @"80", @"Port",
       nil]
     ];
-    
-  NSLog(@"init service");
 
   /* initialize the coder */
   coder = [GWSSOAPCoder new];
@@ -98,16 +95,6 @@
 
   NSLog(@"dict is %d big", [resultDict count]);
   
-  enumerator = [resultDict keyEnumerator];
-/*  while ((key = [enumerator nextObject]))
-  {
-    NSLog(@"%@ - %@", key, [resultDict objectForKey:key]); 
-  } */
-  
-
-//  NSLog(@"request: %@", [[NSString alloc] initWithData:
-//    	[resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
-  
   queryFault = [resultDict objectForKey:@"GWSCoderFault"];
   if (queryFault != nil)
   {
@@ -167,7 +154,7 @@
   if ([[serverUrl substringToIndex:5] isEqualToString:@"https"])
   {
     NSLog(@"we have https....");
-      serverUrl = [@"http" stringByAppendingString:[serverUrl substringFromIndex:5]];
+    serverUrl = [@"http" stringByAppendingString:[serverUrl substringFromIndex:5]];
   }
   
   [coder release];
@@ -184,15 +171,205 @@
   [service setURL:serverUrl];
 }
 
-- (void)query :(NSString *)queryString toWriter:(DBCVSWriter *)writer
+/** <p>Execute SOQL query <i>queryString</i> and returns the resulting DBSObjects as an array.</p>
+  <p>This method will query all resultinng objects of the query, repeatedly querying again if necessary depending on the batch size.</p>
+*/
+- (NSMutableArray *)queryFull :(NSString *)queryString queryAll:(BOOL)all
+{
+  NSString       *qLoc;
+  NSMutableArray *sObjects;
+  
+  sObjects = [[NSMutableArray alloc] init];
+
+  qLoc = [self query: queryString queryAll:all toArray: sObjects];
+  NSLog(@"query loc after first query: %@", qLoc);
+  while (qLoc != nil)
+    qLoc = [self queryMore: qLoc toArray: sObjects];
+  
+  return sObjects;
+}
+
+/** <p>execute SOQL query and write the resulting DBSObjectes into the <i>objects</i> array
+  which must be valid and allocated. </p>
+  <p>If the query locator is returned,  a query more has to be executed.</p>
+*/
+- (NSString *)query :(NSString *)queryString queryAll:(BOOL)all toArray:(NSMutableArray *)objects
+{
+  NSMutableDictionary   *headerDict;
+  NSMutableDictionary   *sessionHeaderDict;
+  NSMutableDictionary   *parmsDict;
+  NSMutableDictionary   *queryParmDict;
+  NSDictionary          *resultDict;
+  NSDictionary          *queryResult;
+  NSDictionary          *result;
+  NSString              *doneStr;
+  BOOL                  done;
+  NSString              *queryLocator;
+  NSArray               *records;
+  NSDictionary          *record;
+  NSDictionary          *queryFault;
+  NSString              *sizeStr;
+  int                   size;
+  
+  /* if the destination array is nil, exit */
+  if (objects == nil)
+    return nil;
+  
+  queryLocator = nil;
+ 
+  /* prepare the header */
+  sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
+  [sessionHeaderDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
+  
+  headerDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
+  [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
+  
+  /* prepare the parameters */
+  queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [queryParmDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
+  [queryParmDict setObject: queryString forKey: @"queryString"];
+  
+  parmsDict = [NSMutableDictionary dictionaryWithCapacity: 1];
+  
+  
+  /* make the query */  
+  if (all)
+    {
+      [parmsDict setObject: queryParmDict forKey: @"queryAll"];
+      [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey];
+      resultDict = [service invokeMethod: @"queryAll"
+			     parameters : parmsDict
+				  order : nil
+				timeout : 90];
+    }
+  else
+    {
+      [parmsDict setObject: queryParmDict forKey: @"query"];
+      [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey];
+      resultDict = [service invokeMethod: @"query"
+			     parameters : parmsDict
+				  order : nil
+				timeout : 90];
+    }
+
+  queryFault = [resultDict objectForKey:@"GWSCoderFault"];
+  if (queryFault != nil)
+    {
+      NSDictionary *fault;
+      NSDictionary *faultDetail;
+    
+      faultDetail = [queryFault objectForKey:@"detail"];
+      fault = [faultDetail objectForKey:@"fault"];
+      NSLog(@"fault: %@", fault);
+      NSLog(@"exception code: %@", [fault objectForKey:@"exceptionCode"]);
+      NSLog(@"exception code: %@", [fault objectForKey:@"exceptionMessage"]);
+    }
+  
+  queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
+  result = [queryResult objectForKey:@"result"];
+  
+  doneStr = [result objectForKey:@"done"];
+  records = [result objectForKey:@"records"];
+  sizeStr = [result objectForKey:@"size"];
+  
+  if (doneStr != nil)
+    {
+      NSLog(@"done: %@", doneStr);
+      done = NO;
+      if ([doneStr isEqualToString:@"true"])
+        done = YES;
+      else if ([doneStr isEqualToString:@"false"])
+        done = NO;
+      else
+        NSLog(@"Done, unexpected value: %@", doneStr);
+    }
+  else
+    {
+      NSLog(@"error, doneStr is nil: unexpected");
+      return nil;
+    }
+  
+  if (sizeStr != nil)
+    {
+      int            i;
+      int            j;
+      int    batchSize;
+      NSMutableArray *keys;
+    
+    
+      size = [sizeStr intValue];
+      NSLog(@"Declared size is: %d", size);
+    
+      /* if we have only one element, put it in an array */
+      if (size == 1)
+        {
+          records = [NSArray arrayWithObject:records];
+        }
+      record = [records objectAtIndex:0];
+      batchSize = [records count];        
+      
+      NSLog(@"records size is: %d", batchSize);
+      /* let's get the fields from the keys of the first record */
+      keys = [NSMutableArray arrayWithArray:[record allKeys]];
+      [keys removeObject:@"GWSCoderOrder"];
+      
+      /* remove some fields which get added automatically by salesforce even if not asked for */
+      [keys removeObject:@"type"];
+      
+      /* remove Id only if it is null, else an array of two populated Id is returned by SF */
+      if (![[record objectForKey:@"Id"] isKindOfClass: [NSArray class]])
+        [keys removeObject:@"Id"];
+      
+      
+      NSLog(@"keys: %@", keys);
+      
+      /* now cycle all the records and read out the fields */
+      for (i = 0; i < batchSize; i++)
+        {
+          DBSObject *sObj;
+        
+          sObj = [[DBSObject alloc] init];
+          record = [records objectAtIndex:i];
+          NSLog(@"record :%@", record);
+          for (j = 0; j < [keys count]; j++)
+            {
+              id       obj;
+              id       value;
+              NSString *key;
+            
+              key = [keys objectAtIndex:j];
+              obj = [record objectForKey: key];
+              if ([key isEqualToString:@"Id"])
+                value = [(NSArray *)obj objectAtIndex: 0];
+              else
+                value = obj;
+              [sObj setValue: value forField: key];
+            }
+          [objects addObject:sObj];
+          [sObj release];
+        }
+    }
+  if (!done)
+    {
+      queryLocator = [result objectForKey:@"queryLocator"];
+      NSLog(@"should do query more, queryLocator: %@", queryLocator);
+    }
+  return queryLocator;
+}
+
+/** <p>Execute SOQL query more and write the resulting DBSObjectes into the <i>objects</i> array
+    which must be valid and allocated, continuing from the given query locator <i>locator</i>. </p>
+    <p>If the query locator is returned,  a query more has to be executed.</p>
+*/
+- (NSString *)queryMore :(NSString *)locator toArray:(NSMutableArray *)objects
 {
   NSMutableDictionary   *headerDict;
   NSMutableDictionary   *sessionHeaderDict;
   NSMutableDictionary   *parmsDict;
   NSMutableDictionary   *queryParmDict;
   NSDictionary          *resultDict;
-  NSEnumerator          *enumerator;
-  NSString              *key;
   NSDictionary          *queryResult;
   NSDictionary          *result;
   NSString              *doneStr;
@@ -204,6 +381,10 @@
   NSString              *sizeStr;
   int                   size;
 
+  /* if the destination array is nil, exit */
+  if (objects == nil)
+    return nil;
+
   /* prepare the header */
   sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
   [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
@@ -212,33 +393,24 @@
   headerDict = [NSMutableDictionary dictionaryWithCapacity: 2];
   [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
   [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
-  
+
   /* prepare the parameters */
   queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
   [queryParmDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
-  [queryParmDict setObject: queryString forKey: @"queryString"];
-  
-  
-  parmsDict = [NSMutableDictionary dictionaryWithCapacity: 1];
-  [parmsDict setObject: queryParmDict forKey: @"query"];
-  [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey];
+  [queryParmDict setObject: locator forKey: @"queryLocator"];  
 
+  parmsDict = [NSMutableDictionary dictionaryWithCapacity: 1];
+  [parmsDict setObject: queryParmDict forKey: @"queryMore"];
+  [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey]; 
   
   /* make the query */  
-  resultDict = [service invokeMethod: @"query"
-                parameters : parmsDict
-		order : nil
-		timeout : 90];
-
-  NSLog(@"dict is %d big", [resultDict count]);
-
-  enumerator = [resultDict keyEnumerator];
-  NSLog(@"before cleaning");
-  while ((key = [enumerator nextObject]))
-  {
-    NSLog(@"%@ - %@", key, [resultDict objectForKey:key]); 
-  }
+  resultDict = [service invokeMethod: @"queryMore"
+                         parameters : parmsDict
+                              order : nil
+                            timeout : 90];
   
+
+  queryLocator = nil;
   queryFault = [resultDict objectForKey:@"GWSCoderFault"];
   if (queryFault != nil)
     {
@@ -254,10 +426,8 @@
 
   queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
   result = [queryResult objectForKey:@"result"];
-//  NSLog(@"result: %@", result);
 
   doneStr = [result objectForKey:@"done"];
-  queryLocator = [result objectForKey:@"queryLocator"];
   records = [result objectForKey:@"records"];
   sizeStr = [result objectForKey:@"size"];
  
@@ -275,7 +445,7 @@
   else
     {
       NSLog(@"error, doneStr is nil: unexpected");
-      return;
+      return nil;
     }
 
   if (sizeStr != nil)
@@ -284,7 +454,6 @@
       int            j;
       int    batchSize;
       NSMutableArray *keys;
-      NSMutableArray *set;
       
       
       size = [sizeStr intValue];
@@ -294,7 +463,6 @@
       if (size == 1)
         {
           records = [NSArray arrayWithObject:records];
-          batchSize = 1;
         }
       record = [records objectAtIndex:0];
       batchSize = [records count];        
@@ -315,17 +483,14 @@
 
       NSLog(@"keys: %@", keys);
       
-      [writer setFieldNames:[NSArray arrayWithArray:keys] andWriteIt:YES];
-      
-      set = [[NSMutableArray alloc] init];
-      
       /* now cycle all the records and read out the fields */
       for (i = 0; i < batchSize; i++)
         {
-          NSMutableArray *values;
+          DBSObject *sObj;
 	  
+          sObj = [[DBSObject alloc] init];
           record = [records objectAtIndex:i];
-          values = [NSMutableArray arrayWithCapacity:[keys count]];
+ 
           for (j = 0; j < [keys count]; j++)
             {
               id       obj;
@@ -338,171 +503,93 @@
                   value = [(NSArray *)obj objectAtIndex: 0];
               else
                   value = obj;
-              [values addObject:value];
+              [sObj setValue: value forField: key];
             }
-//          NSLog(@"%d: %@", i, values);
-          [set addObject:values];
+          [objects addObject:sObj];
+          [sObj release];
         }
-      [writer writeDataSet:set];
     }
   if (!done)
     {
+      queryLocator = [result objectForKey:@"queryLocator"];
       NSLog(@"should do query more, queryLocator: %@", queryLocator);
-      [self queryMore :queryLocator toWriter:writer];
     }
+  return queryLocator;
 }
 
-- (void)queryMore :(NSString *)queryLocator toWriter:(DBCVSWriter *)writer
+- (void)query :(NSString *)queryString queryAll:(BOOL)all toWriter:(DBCVSWriter *)writer
 {
-  NSMutableDictionary   *headerDict;
-  NSMutableDictionary   *sessionHeaderDict;
-  NSMutableDictionary   *parmsDict;
-  NSMutableDictionary   *queryParmDict;
-  NSDictionary          *resultDict;
-  NSEnumerator          *enumerator;
-  NSString              *key;
-  NSDictionary          *queryResult;
-  NSDictionary          *result;
-  NSString              *doneStr;
-  BOOL                  done;
-  NSString              *nextQueryLocator;
-  NSArray               *records;
-  NSDictionary          *record;
-  NSString              *sizeStr;
-  int                   size;
+  int            i;
+  int            j;
+  int            batchSize;
 
-  /* prepare the header */
-  sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
-  [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
-  [sessionHeaderDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
-
-  headerDict = [NSMutableDictionary dictionaryWithCapacity: 2];
-  [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
-  [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
-  
-  /* prepare the parameters */
-  queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
-  [queryParmDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
-  [queryParmDict setObject: queryLocator forKey: @"queryLocator"];
+  NSString       *qLoc;
+  NSMutableArray *sObjects;
+  NSArray        *keys;
   
   
-  parmsDict = [NSMutableDictionary dictionaryWithCapacity: 1];
-  [parmsDict setObject: queryParmDict forKey: @"queryMore"];
-  [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey];
+  sObjects = [[NSMutableArray alloc] init];
 
-  
-  /* make the query */  
-  resultDict = [service invokeMethod: @"queryMore"
-                parameters : parmsDict
-                order : nil
-              timeout : 90];
+  qLoc = [self query: queryString queryAll: all toArray: sObjects];
 
-  NSLog(@"dict is %d big", [resultDict count]);
-
-  enumerator = [resultDict keyEnumerator];
-  while ((key = [enumerator nextObject]))
-  {
-    NSLog(@"%@ - %@", key, [resultDict objectForKey:key]); 
-  }
-  
-  queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
-  result = [queryResult objectForKey:@"result"];
-
-  doneStr = [result objectForKey:@"done"];
-  nextQueryLocator = [result objectForKey:@"queryLocator"];
-  records = [result objectForKey:@"records"];
-  sizeStr = [result objectForKey:@"size"];
- 
-  if (doneStr != nil)
+  keys = nil;
+  batchSize = [sObjects count];
+  if (batchSize > 0)
     {
-      NSLog(@"done: %@", doneStr);
-      done = NO;
-      if ([doneStr isEqualToString:@"true"])
-        done = YES;
-      else if ([doneStr isEqualToString:@"false"])
-        done = NO;
-      else
-        NSLog(@"Done, unexpected value: %@", doneStr);
-    }
-  else
-    {
-      NSLog(@"error, doneStr is nil: unexpected");
-      return;
-    }
-
-  if (sizeStr != nil)
-    {
-      int            i;
-      int            j;
-      int    batchSize;
-      NSMutableArray *keys;
+      DBSObject *obj;
       NSMutableArray *set;
-      
-      
-      size = [sizeStr intValue];
-      NSLog(@"Declared size is: %d", size);
-      
-      /* if we have only one element, put it in an array */
-      if (size == 1)
-        {
-          records = [NSArray arrayWithObject:records];
-          batchSize = 1;
-        }
-      record = [records objectAtIndex:0];
-      batchSize = [records count];        
-      
-      NSLog(@"records size is: %d", batchSize);
-      
-      /* let's get the fields from the keys of the first record */
-      keys = [NSMutableArray arrayWithArray:[record allKeys]];
-      [keys removeObject:@"GWSCoderOrder"];
 
-      /* remove some fields which get added automatically by salesforce even if not asked for */
-      [keys removeObject:@"type"];
-      
-      /* remove Id only if it is null, else an array of two populated Id is returned by SF */
-      if (![[record objectForKey:@"Id"] isKindOfClass: [NSArray class]])
-        [keys removeObject:@"Id"];
-      
-      NSLog(@"keys: %@", keys);
-      
+      obj = [sObjects objectAtIndex: 0];
+      keys = [obj fieldNames];
+      [writer setFieldNames:keys andWriteIt:YES];
 
       set = [[NSMutableArray alloc] init];
-      
-      /* now cycle all the records and read out the fields */
       for (i = 0; i < batchSize; i++)
         {
           NSMutableArray *values;
+	  DBSObject *record;
 	  
-          record = [records objectAtIndex:i];
+          record = [sObjects objectAtIndex:i];
           values = [NSMutableArray arrayWithCapacity:[keys count]];
           for (j = 0; j < [keys count]; j++)
-          {
-              id       obj;
-              id       value;
-              NSString *key;
-              
-              key = [keys objectAtIndex:j];
-              obj = [record objectForKey: key];
-              if ([key isEqualToString:@"Id"])
-                  value = [(NSArray *)obj objectAtIndex: 0];
-              else
-                  value = obj;
-              [values addObject:value];
-          }
-//          NSLog(@"%d: %@", i, values);
+            {
+              [values addObject:[record fieldValue: [keys objectAtIndex:j]]];
+            }
           [set addObject:values];
         }
       [writer writeDataSet:set];
-    }
-  if (!done)
-    {
-      NSLog(@"should do query more, nextQueryLocator: %@", nextQueryLocator);
-      [self queryMore :nextQueryLocator toWriter:writer];
+      [set release];
     }
 
+  while (qLoc != nil)
+    {
+      NSMutableArray *set;
+
+      [sObjects removeAllObjects];
+      NSLog(@"size %d", [sObjects count]);
+      qLoc = [self queryMore: qLoc toArray: sObjects];
+      batchSize = [sObjects count];
+      set = [[NSMutableArray alloc] init];
+      for (i = 0; i < batchSize; i++)
+        {
+          NSMutableArray *values;
+	  DBSObject *record;
+	  
+          record = [sObjects objectAtIndex:i];
+          values = [NSMutableArray arrayWithCapacity:[keys count]];
+          for (j = 0; j < [keys count]; j++)
+            {
+              [values addObject:[record fieldValue: [keys objectAtIndex:j]]];
+            }
+          [set addObject:values];
+        }
+      [writer writeDataSet:set];
+      [set release];
+      
+    }
 }
 
+
 - (void)create :(NSString *)objectName fromReader:(DBCVSReader *)reader
 {
   NSMutableDictionary   *headerDict;
@@ -514,13 +601,11 @@
   NSString              *key;
   NSDictionary          *queryResult;
   NSDictionary          *result;
-  NSString              *doneStr;
   NSArray               *records;
   NSDictionary          *record;
   NSDictionary          *queryFault;
   NSString              *sizeStr;
   int                   size;
-  NSMutableDictionary   *objectsDict;
   NSArray               *objectsArray;
   NSArray               *fieldValues;
   NSArray               *fieldNames;
@@ -533,7 +618,6 @@
   /* first the fields */
   fieldNames = [reader fieldNames];
   fieldCount = [fieldNames count];
-  objectsDict = [NSMutableDictionary dictionaryWithCapacity: 2];
   objectsArray = [reader readDataSet];
   
   
@@ -599,8 +683,7 @@
 		order : nil
 		timeout : 90];
   
-  NSLog(@"request: %@", [[NSString alloc] initWithData:
-    	[resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
+//  NSLog(@"request: %@", [[NSString alloc] initWithData: [resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
   
 
   NSLog(@"dict is %d big", [resultDict count]);
@@ -629,7 +712,6 @@
   result = [queryResult objectForKey:@"result"];
   NSLog(@"result: %@", result);
 
-  doneStr = [result objectForKey:@"done"];
   records = [result objectForKey:@"records"];
   sizeStr = [result objectForKey:@"size"];
  
@@ -675,6 +757,8 @@
           NSLog(@"%d: %@", i, values);
           [set addObject:values];
         }
+      /* we don't do yet anything useful with the results... */
+      [set release];
     }
 }
 
@@ -689,13 +773,11 @@
   NSString              *key;
   NSDictionary          *queryResult;
   NSDictionary          *result;
-  NSString              *doneStr;
   NSArray               *records;
   NSDictionary          *record;
   NSDictionary          *queryFault;
   NSString              *sizeStr;
   int                   size;
-  NSMutableDictionary   *objectsDict;
   NSArray               *objectsArray;
   NSArray               *fieldValues;
   NSArray               *fieldNames;
@@ -708,7 +790,6 @@
   /* first the fields */
   fieldNames = [reader fieldNames];
   fieldCount = [fieldNames count];
-  objectsDict = [NSMutableDictionary dictionaryWithCapacity: 2];
   objectsArray = [reader readDataSet];
   
   
@@ -774,8 +855,8 @@
                               order : nil
                             timeout : 90];
   
-  NSLog(@"request: %@", [[NSString alloc] initWithData:
-    	[resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
+//  NSLog(@"request: %@", [[NSString alloc] initWithData:
+//    	[resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
   
   
   NSLog(@"dict is %d big", [resultDict count]);
@@ -804,7 +885,6 @@
   result = [queryResult objectForKey:@"result"];
   NSLog(@"result: %@", result);
   
-  doneStr = [result objectForKey:@"done"];
   records = [result objectForKey:@"records"];
   sizeStr = [result objectForKey:@"size"];
   
@@ -850,6 +930,8 @@
       NSLog(@"%d: %@", i, values);
       [set addObject:values];
       }
+      /* we don't do yet anything useful with the results... */
+      [set release];
     }
 }
 
@@ -861,13 +943,9 @@
   NSMutableDictionary   *parmsDict;
   NSMutableDictionary   *queryParmDict;
   NSDictionary          *resultDict;
-  NSEnumerator          *enumerator;
-  NSString              *key;
   NSDictionary          *queryResult;
   NSDictionary          *result;
-  NSString              *encoding;
   NSDictionary          *queryFault;
-  NSString              *maxBatchSizeStr;
   NSArray               *types;
 
   /* prepare the header */
@@ -915,13 +993,150 @@
   result = [queryResult objectForKey:@"result"];
   NSLog(@"result: %@", result);
 
-  encoding = [result objectForKey:@"encoding"];
-  maxBatchSizeStr = [result objectForKey:@"maxBatchSize"];
   types = [result objectForKey:@"types"];
 
   return types;
 }
 
+- (void)describeSObject: (NSString *)objectType toWriter:(DBCVSWriter *)writer
+{
+  int            i;
+  int            size;
+  DBSObject      *object;
+  NSDictionary   *properties;
+  NSArray        *fields;
+  NSArray        *keys;
+  NSMutableArray *set;
+
+  
+  object = [self describeSObject: objectType];
+  fields = [object fieldNames];
+  size = [fields count];
+  
+  if (size < 1)
+    return;
+  
+  keys = [[object propertiesOfField: [fields objectAtIndex: 0]] allKeys];
+  [writer setFieldNames:[NSArray arrayWithArray:keys] andWriteIt:YES];
+  
+  set = [[NSMutableArray alloc] init];
+  
+  for (i = 0; i < size; i++)
+    {
+      NSMutableArray *values;
+      int            j;
+      NSString       *field;
+      
+      field = [fields objectAtIndex: i];
+      properties = [object propertiesOfField: field];
+      values = [NSMutableArray arrayWithCapacity:[keys count]];
+      for (j = 0; j < [keys count]; j++)
+        {
+          id       obj;
+          id       value;
+          NSString *key;
+      
+          key = [keys objectAtIndex:j];
+          obj = [properties objectForKey: key];
+      
+          value = obj;
+          [values addObject:value];
+        }
+      [set addObject:values];
+    }
+  [writer writeDataSet:set];
+  [set release];
+}
+
+- (DBSObject *)describeSObject: (NSString *)objectType
+{
+  NSMutableDictionary   *headerDict;
+  NSMutableDictionary   *sessionHeaderDict;
+  NSMutableDictionary   *parmsDict;
+  NSMutableDictionary   *queryParmDict;
+  NSDictionary          *resultDict;
+  NSDictionary          *queryResult;
+  NSDictionary          *result;
+  NSDictionary          *queryFault;
+  NSArray               *records;
+  NSDictionary          *record;
+  int                   i;
+  int                   size;
+  NSMutableArray        *keys;
+  DBSObject             *object;
+
+
+  /* prepare the header */
+  sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
+  [sessionHeaderDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
+
+  headerDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
+  [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
+  
+  /* prepare the parameters */
+  queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
+  [queryParmDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
+
+  [queryParmDict setObject: objectType forKey: @"sObjectType"];
+
+  
+  parmsDict = [NSMutableDictionary dictionaryWithCapacity: 1];
+  [parmsDict setObject: queryParmDict forKey: @"describeSObject"];
+  [parmsDict setObject: headerDict forKey:GWSSOAPMessageHeadersKey];
+  
+  /* make the query */  
+  resultDict = [service invokeMethod: @"describeSObject"
+                parameters : parmsDict
+		order : nil
+		timeout : 90];
+  
+  queryFault = [resultDict objectForKey:@"GWSCoderFault"];
+  if (queryFault != nil)
+    {
+      NSDictionary *fault;
+      NSDictionary *faultDetail;
+
+      faultDetail = [queryFault objectForKey:@"detail"];
+      fault = [faultDetail objectForKey:@"fault"];
+      NSLog(@"fault: %@", fault);
+      NSLog(@"exception code: %@", [fault objectForKey:@"exceptionCode"]);
+      NSLog(@"exception message: %@", [fault objectForKey:@"exceptionMessage"]);
+      [[NSException exceptionWithName:@"DBException" reason:[fault objectForKey:@"exceptionMessage"] userInfo:nil] raise];
+    }
+
+  queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
+  result = [queryResult objectForKey:@"result"];
+
+  records = [result objectForKey:@"fields"];
+  size = [records count];
+
+  /* if we have only one element, put it in an array */
+  if (size == 1)
+    records = [NSArray arrayWithObject:records];
+  record = [records objectAtIndex:0];    
+ 
+
+  keys = [NSMutableArray arrayWithArray:[record allKeys]];
+  [keys removeObject:@"GWSCoderOrder"];
+
+  object = [[DBSObject alloc] init];
+
+  for (i = 0; i < size; i++)
+    {
+      NSMutableDictionary *props;
+      NSString *fieldName;
+      
+      record = [records objectAtIndex:i];
+      props = [NSMutableDictionary dictionaryWithDictionary: record];
+      [props removeObjectForKey:@"GWSCoderOrder"];
+      fieldName = [props objectForKey: @"name"];
+      [object setProperties:[NSDictionary dictionaryWithDictionary: props] forField: fieldName];
+    }
+    return [object autorelease];
+}
+
 
 - (NSMutableArray *)delete :(NSArray *)objectIdArray
 {
@@ -930,18 +1145,10 @@
   NSMutableDictionary   *parmsDict;
   NSMutableDictionary   *queryParmDict;
   NSDictionary          *resultDict;
-  NSEnumerator          *enumerator;
-  NSString              *key;
   NSDictionary          *queryResult;
   NSDictionary          *result;
-  NSString              *doneStr;
-  NSArray               *records;
-  NSDictionary          *record;
   NSDictionary          *queryFault;
-  NSString              *sizeStr;
-  int                   size;
   NSMutableArray        *queryObjectsDict;
-  NSString              *errorStr;
   NSMutableArray        *resultArray;
   
   /* prepare the header */
@@ -971,12 +1178,6 @@
                 parameters : parmsDict
 		order : nil
 		timeout : 90];
-  
-  NSLog(@"request: %@", [[NSString alloc] initWithData:
-    	[resultDict objectForKey:@"GWSCoderRequestData"] encoding: NSUTF8StringEncoding]);
-  
-
-  NSLog(@"dict is %d big", [resultDict count]);
 
   queryFault = [resultDict objectForKey:@"GWSCoderFault"];
   if (queryFault != nil)
@@ -999,7 +1200,6 @@
 
   if (result != nil)
     {
-      NSMutableArray *keys;
       id resultRow;
       NSEnumerator   *objEnu;
       NSDictionary   *rowDict;
